/*
 * @plotXYFrame.java Version 1.1 05/25/2011
 *
 * Copyright (c) 2011 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import javax.swing.*;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Observable;

import plot.plotXYData;
import plot.plotXYStruct;
import plot.plotSymbolStruct;

import iqstrat.iqstratHeadersListStruct;

import quake.quakeSensorsStruct;         // Sensor Data Structure

import util.utilLandGridListStruct;      // TRS Land Grid List Structure

/** Class plotXYFrame
 *  <p> This class will display the Cross Plot Frame
 *
 *  @version 1.1 05/25/2011
 *  @author  John R. Victorine
 */

public class plotXYFrame extends JFrame //implements ActionListener
{
  private Observable               notifier = null; // Observalbles Object
  private int                      iPlot    = -1;
  private plotXYStruct             stPlot   = null;
  private plotXYData               panel    = null;
  private iqstratHeadersListStruct stWells  = null;
  private quakeSensorsStruct       stSensor = null; // CO2 Sensors Location Structure
  private utilLandGridListStruct   stGrid   = null; // TRS Land Grid List Structure

  /** Constructor plotXYFrame()
   *  <p> This is the constructor for this class
   *  @param notifier = Oberservable
   *  @param iPlot    = Plot Type
   *  @param stPlot   = XY Plot Data Structure
   *  @param stWells  = SWD Wells & Wellington KGS Wells
   */

  public plotXYFrame( Observable notifier,
                      int iPlot,  plotXYStruct stPlot,
                      iqstratHeadersListStruct stWells )
  {
    try
    {
	  this.notifier = notifier;
	  this.iPlot    = iPlot;
      this.stPlot   = stPlot;
      this.stWells  = stWells;

      jbInit();

      addWindowListener( new plotXYFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Constructor plotXYFrame()
   *  <p> This is the constructor for this class
   *  @param notifier = Oberservable
   *  @param iPlot    = Plot Type
   *  @param stPlot   = XY Plot Data Structure
   *  @param stWells  = SWD Wells & Wellington KGS Wells
   *  @param stSensor = CO2 Sensor Location Structure
   *  @param stGrid   = TRS Land Grid List Structure
   */

  public plotXYFrame( Observable notifier,
                      int iPlot,  plotXYStruct stPlot,
                      iqstratHeadersListStruct stWells,
                      quakeSensorsStruct       stSensor,
                      utilLandGridListStruct   stGrid )
  {
    try
    {
	  this.notifier = notifier;
	  this.iPlot    = iPlot;
      this.stPlot   = stPlot;
      this.stWells  = stWells;
      this.stSensor = stSensor;
      this.stGrid   = stGrid;

      jbInit();

      addWindowListener( new plotXYFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Frame
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
    JScrollPane jScrollPane = new JScrollPane();

    this.getContentPane().setLayout(new BorderLayout());
    this.setTitle("KS Earthquake 2D Plot");

    panel = new plotXYData( iPlot, stPlot, stWells, stSensor, stGrid );

    this.getContentPane().add(jScrollPane, BorderLayout.CENTER);
    jScrollPane.getViewport().add(panel, null);

    // Add a Window Listener to close dialog

    this.addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e) { close(); }
    });

    // Set Dimension of Frame and Display it.

//    this.setSize(new Dimension(700, 500));
    this.setSize(new Dimension(900, 700));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
//    this.setLocation(3*(d.width - this.getSize().width) / 4,
//                     (d.height - this.getSize().height) / 2);
//    this.setLocation(575, (d.height - this.getSize().height) / 2);

//    this.setLocation( (d.width - this.getSize().width)/2,
//                      (d.height - this.getSize().height)/2);

    this.setLocation( 10, (d.height - this.getSize().height) - 50);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** Method refresh()
   * <p> This method will refresh the plot if it exists
   */

  public void refresh() { panel.repaint(); }

  /* ================================================================== *
   * ------------------------- CLOSE METHODS -------------------------- *
   * ================================================================== */

  /** METHOD plotXYFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class plotXYFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
	  if (notifier != null)
        notifier.notifyObservers(new String("Close Plot Control"));
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
	notifier = null;
    stPlot   = null;
    stWells  = null;
    stSensor = null;
    stGrid   = null;

    if (panel != null)
      panel.close();
    panel     = null;

    dispose();
  }


  /* ================================================================== *
   * -------------------------- GET METHODS --------------------------- *
   * ================================================================== */

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (panel != null) image = panel.getImage();

    return (image);
  }

  /* ================================================================== *
   * -------------------------- SET METHODS --------------------------- *
   * ================================================================== */

  /** Method setLegend()
   *  <p> The Legend for the XY Plot.
   *  @param iLegend  - Number of Legend Rows.
   *  @param stLegend - Array of Legends.
   */

  public void setLegend(int iLegend, plotSymbolStruct stLegend[])
  {
    panel.setLegend(iLegend, stLegend);
  }

  /** Method setData()
   *  <p> This is the Constructor for this class.
   *  @param stPlot - Plot Data Structure.
   */

  public void setData(plotXYStruct stPlot)
  {
    if (panel != null)  { panel.setData(stPlot); }
  }

  /** Method setFluid()
   *  <p> This method will set the Fluid type and recompute the marker minerals
   */

  public void setFluid( int iFluid )
  {
	if (panel != null)  { panel.setFluid( iFluid ); }
  }

  /** Method setTriangle()
   * <p> This method will set the triangle data
   * @param iData      = The total number points
   * @param sLithology = The lithology values
   */

  public void setTriangle(int iData, String sLithology[])
  {
    if (panel != null) { panel.setTriangle(iData, sLithology); }
  }

  /** Method setFlag()
   *  <p> Set a Flag to filter extra data or not.
   *  @param bFlag = true to filter data or false not to filter extra data.
   */

  public void setFlag(boolean bFlag) { panel.setFlag(bFlag); }

  /** Method setTitle1()
   * <p> This method will set the first user title
   * @param sText = the first title
   */

  public void setTitle1(String sText) { panel.setTitle1(sText); }

  /** Method setTitle2()
   * <p> This method will set the second user title
   * @param sText = the second title
   */

  public void setTitle2(String sText) { panel.setTitle2(sText); }

  /** Method setTitle3()
   * <p> This method will set the third user title
   * @param sText = the third title
   */

  public void setTitle3(String sText) { panel.setTitle3(sText); }

  /** Method setXAxis()
   * <p> This method will reset the X-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setXAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    panel.setXAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /** Method setYAxis()
   * <p> This method will reset the Y-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setYAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    panel.setYAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /** Method setStatus()
   *  <p> Turn the Wells on or off.
   *  @param iAction = 0 wells off, 1 wells on.
   */

  public void setStatus(int iAction) { panel.setStatus(iAction); }

  /** Method setGrid()
   *  <p> Turn the TRS Grid on or off.
   *  @param iTRS = 0 wells off, 1 wells on.
   */

  public void setGrid(int iTRS) { panel.setGrid(iTRS); }
}

/*
 *  @version 1.1 05/25/2011
 *  @author  John Victorine
 */
